@@ -33,7 +33,8 @@ typedef struct _GOptionEntry   GOptionEntry;
 typedef enum
 {
   G_OPTION_FLAG_HIDDEN       = 1 << 0,
-  G_OPTION_FLAG_IN_MAIN      = 1 << 1
+  G_OPTION_FLAG_IN_MAIN      = 1 << 1,
+  G_OPTION_FLAG_REVERSE	     = 1 << 2
 } GOptionFlags;
 
 typedef enum
